import sys
from typing import List
import numpy as np
from setuptools import Extension


if sys.platform == 'win32':
    extra_compile_args = ['/openmp']
    extra_link_args = None
else:
    # linux
    extra_compile_args = ['-fopenmp']
    extra_link_args = ['-fopenmp']


def make_ext(modname: str, sources: List[str], **kwargs) -> Extension:
    """
    Make Extension for module using the given sources
    uses the platform specific compile and link arguments to use openmp

    Parameters
    ----------
    modname:
        the name of the extension module to build
    sources:
        list of source filenames, relative to the distribution root
        (where the setup script lives), in Unix form (slash-separated)
        for portability
    """
    if not isinstance(sources, list):
        sources = [sources]
    extension = Extension(modname,
                          sources=sources,
                          extra_compile_args=extra_compile_args,
                          extra_link_args=extra_link_args,
                          include_dirs=[np.get_include()],
                          **kwargs,
                          )
    return extension



# -*- coding: utf-8 -*-

import os
from typing import List, Dict
from setuptools import Extension
import numpy as np
from Cython.Build import cythonize

from .build_config import make_ext


def make_extensions(ext_modnames: List[str],
                    further_args: Dict[str, str]={},
                    source_dir: str='src',
                    annotate: bool=True,
                    linetrace: bool=True,
                    language_level: str='3') -> List[Extension]:
    """
    add sources to the ext_modules specified in the input list

    Parameters
    ----------
    ext_modnames :
        the extension modules to create
    further_args : dict with further arguments for extension module
    source_dir :
        the source directory relative to the setup.py file
    annotate:
        if true, the code is annotated
    linetrace:
        if true, the compiler-directive linetrace is enabled
    language_level:
        the compiler-directive for the language_level (2 or 3)

    Returns
    -------
    :
        list of Extension-instances
    """
    extensions = []


    suffix = '.pyx'
    for modname in ext_modnames:
        mn = modname.split('.')
        pyxfilename = os.path.join(source_dir, *mn) + suffix
        sources = [pyxfilename]
        further_arg = further_args.get(modname, {})
        if further_arg:
            more_sources = further_arg.pop('sources')
            ms = mn[:-1]
            full_more_sources = []
            for source in more_sources:
                full_source = [source_dir] + ms + [source]
                full_more_sources.append(os.sep.join(full_source))
            sources.extend(full_more_sources)
        extension = make_ext(modname, sources, **further_arg)
        extensions.append(extension)

    cython_extensions = cythonize(extensions,
                                  annotate=annotate,
                                  compiler_directives={
                                      'linetrace': linetrace,
                                      'language_level': language_level,
                                      },
                                  )
    return cython_extensions


from pathlib import Path
import sys
src = Path(__file__).parent.joinpath('src')
sys.path.append(str(src))

from setuptools import setup
from cythonarrays.make_cython_extensions import make_extensions


ext_modnames = ['cythonarrays.array_shapes',
                'cythonarrays.tests.example_cython',
                'cythonarrays.tests.simple_cython',
                ]

package_name = "cythonarrays"

setup(
    ext_modules=make_extensions(ext_modnames),
)


# -*- coding: utf-8 -*-

from cythonarrays.configure_logger import (SimLogger,
                                           get_module_logger,
                                           get_logger)
import os
import glob
from .module_with_logging import function_with_logging
from cythonarrays.build_config import make_ext

class Test01_Logger:
    """Test the the SimLogger"""
    def test_01_log_something(self, tmpdir: str):
        """log something to the logger"""
        print(tmpdir)
        scenario = 'TestScenario_without_Packages'
        sim_logger = SimLogger()

        sim_logger.configure(LOG_FOLDER=tmpdir, scenario=scenario)

        # create a logger for the 'cythonarrays'-package
        logger = get_logger(self)
        # assert that cythonarrays is registred now
        assert __package__.split('.')[0] in sim_logger.packages

        # truncate loggers
        sim_logger.truncate_packages()

        # assert that there are no packages registred with the logger
        assert not sim_logger.packages

        function_with_logging('Log something in function_with_logging without package')

        # assert that cythonarrays is registred now
        assert __package__.split('.')[0] in sim_logger.packages

        logger = get_logger(self)
        logger.info('Info in test_01')
        logger.debug('Debug in test_01')
        logger.warning('Warn in test_01')

        logger = get_logger(make_ext)
        logger.warning('Logging for a make_ext function')

        instance = 'A String'
        logger = get_logger(instance)
        logger.info('Logging for an str-instance form builtins without module')

        print(sim_logger.packages)
        # assert that cythonarrays is still registred
        assert __package__.split('.')[0] in sim_logger.packages

        logfiles = glob.glob(os.path.join(tmpdir, f'{scenario}*.log'))
        assert logfiles
        print(logfiles)
        for logfile in logfiles:
            with open(logfile) as f:
                for line in f:
                    print(line.strip())

    def test_03_module_logger(self, tmpdir):
        """Test the module logger"""
        module_logger = get_module_logger(self.__module__)
        module_logger.info('Info in Module {}'.format(self.__module__))
